/**
 * Copyright or © or Copr. IETR/INSA: Maxime Pelcat, Jean-François Nezan,
 * Karol Desnos, Julien Heulot
 * 
 * [mpelcat,jnezan,kdesnos,jheulot]@insa-rennes.fr
 * 
 * This software is a computer program whose purpose is to prototype
 * parallel applications.
 * 
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
package org.ietr.preesm.codegen.xtend.printer

import org.ietr.preesm.codegen.xtend.model.codegen.CoreBlock
import org.ietr.preesm.codegen.xtend.model.codegen.Delimiter
import org.ietr.preesm.codegen.xtend.model.codegen.Direction
import org.ietr.preesm.codegen.xtend.model.codegen.SharedMemoryCommunication

/**
 * This {@link ComTrackingPrinter} is a dummy implementation of the 
 * {@link DefaultPrinter} where all print methods print nothing
 * except for communications. The goal is to track the order of communication
 * calls for debug purpose.
 * @author mpelcat
 */
class ComTrackingPrinter extends DefaultPrinter {
	
		override printCoreBlockHeader(CoreBlock block) '''
			/** 
			 * @file «block.name».txt
			 * @generated by «this.class.simpleName» to check communications
			 */
			
	'''
	
	/**
	 * Printing reports for communications.
	 * @see CodegenAbstractPrinter#preProcessing(List,List)
	 */
	override printSharedMemoryCommunication(SharedMemoryCommunication communication) '''
	«/*on each communication start, we track the id and cores*/
	»«IF communication.delimiter == Delimiter::START
	»«IF communication.direction == Direction::SEND»send«
	ELSEIF communication.direction == Direction::RECEIVE»recv«
	ENDIF»(«
	communication.sendStart.coreContainer.name»->«
	communication.receiveStart.coreContainer.name») com «
	communication.id» «communication.data.name»
	«ENDIF»
	'''

	
}
