/*
 * Copyright (c) 2008, IETR/INSA of Rennes
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *   * Neither the name of the IETR/INSA of Rennes nor the names of its
 *     contributors may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.ietr.dftools.graphiti.ui.figure;

import org.eclipse.draw2d.EllipseAnchor;
import org.eclipse.draw2d.geometry.Point;

/**
 * This class provides a connection anchor for ellipses.
 * 
 * @author Jonathan Piat
 * @author Matthieu Wipliez
 * 
 */
public class EllipsePortAnchor extends EllipseAnchor {

	private PortAnchorReferenceManager mgr;

	/**
	 * Creates a new ellipse port anchor.
	 * 
	 * @param figure
	 *            The owning vertex figure.
	 * @param portName
	 *            The port name associated with this connection anchor.
	 * @param isOutput
	 *            Whether the connection is input (false) or output (true).
	 */
	public EllipsePortAnchor(VertexFigure figure, String portName,
			boolean isOutput) {
		super(figure);
		mgr = new PortAnchorReferenceManager(figure, portName, isOutput);
	}

	@Override
	public Point getLocation(Point reference) {
		Point mgrReference = mgr.getReferencePoint(this);
		if (mgrReference == null) {
			return super.getLocation(reference);
		} else {
			return mgrReference;
		}
	}

	@Override
	public Point getReferencePoint() {
		Point reference = mgr.getReferencePoint(this);
		if (reference == null) {
			return super.getReferencePoint();
		} else {
			return reference;
		}
	}
}
